#include <windows.h>
#include <tchar.h>

#include <setupapi.h>
#include <initguid.h>

#include <stdio.h>

DEFINE_GUID(GUID_DEVINTERFACE_GX3, 0x2c9213e2, 0xac26, 0x485c, 0x8e, 0x26, 0xc, 0x32, 0x21, 0x35, 0x34, 0x7);

#define IOCTL_INDEX             0x800
#define FILE_DEVICE_GX3         0x65500

#define IOCTL_GX3_GET_METRICS   CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX, 	    METHOD_BUFFERED, 	FILE_READ_ACCESS)
#define IOCTL_RAW_SET_BACKLIGHT CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX + 1, 	METHOD_BUFFERED, 	FILE_WRITE_ACCESS)
#define IOCTL_RAW_SET_CONTRAST CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX + 2, 	METHOD_BUFFERED, 	FILE_WRITE_ACCESS)
#define IOCTL_RAW_GET_SPLASH CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX + 4, 	METHOD_BUFFERED, 	FILE_READ_ACCESS)
#define IOCTL_RAW_GET_FRAME CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX + 5, 	METHOD_BUFFERED, 	FILE_READ_ACCESS)
#define IOCTL_RAW_PUT_FRAME CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX + 6, 	METHOD_BUFFERED, 	FILE_WRITE_ACCESS)
#define IOCTL_RAW_SET_DEFAULT_BACKLIGHT CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX + 7, 	METHOD_BUFFERED, 	FILE_WRITE_ACCESS)
#define IOCTL_RAW_SET_DEFAULT_CONTRAST CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX + 8, 	METHOD_BUFFERED, 	FILE_WRITE_ACCESS)
#define IOCTL_SESSION_CREATE           CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX + 9, 	METHOD_BUFFERED, 	FILE_ANY_ACCESS)
#define IOCTL_SESSION_CLOSE            CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX + 10, 	METHOD_BUFFERED, 	FILE_ANY_ACCESS)
#define IOCTL_SESSION_SEND_FRAME       CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX + 11, 	METHOD_BUFFERED, 	FILE_WRITE_ACCESS)
#define IOCTL_SESSION_SET_CONTRAST     CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX + 12, 	METHOD_BUFFERED, 	FILE_WRITE_ACCESS)
#define IOCTL_SESSION_SET_BACKLIGHT    CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX + 13, 	METHOD_BUFFERED, 	FILE_WRITE_ACCESS)
#define IOCTL_SESSION_LIST_SESSIONS    CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX + 14, 	METHOD_BUFFERED, 	FILE_READ_ACCESS)
#define IOCTL_GET_FW_REV               CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX + 15, 	METHOD_BUFFERED, 	FILE_READ_ACCESS)
#define IOCTL_GET_SERIAL               CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX + 16, 	METHOD_BUFFERED, 	FILE_READ_ACCESS)
#define IOCTL_GET_DEFAULT_BACKLIGHT CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX + 17, 	METHOD_BUFFERED, 	FILE_READ_ACCESS)
#define IOCTL_GET_DEFAULT_CONTRAST CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX + 18,  METHOD_BUFFERED, 	FILE_READ_ACCESS)
#define IOCTL_START_FWU            CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX + 19, 	METHOD_BUFFERED, 	FILE_ANY_ACCESS)
#define IOCTL_GET_IR_STATE            CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX + 20, 	METHOD_BUFFERED, 	FILE_READ_ACCESS)
#define IOCTL_SET_IR_STATE            CTL_CODE(FILE_DEVICE_GX3, 	IOCTL_INDEX + 21, 	METHOD_BUFFERED, 	FILE_WRITE_ACCESS)

int CreateSession(HANDLE handle, char* Data)
{
	DWORD BytesReturned = 0;
	int SessionID = 0;
	DeviceIoControl(handle, IOCTL_SESSION_CREATE, Data, strlen(Data), &SessionID, 4, &BytesReturned, NULL);
	return SessionID;
}

void SendFrame(HANDLE handle, int Session, BYTE* Data)
{
	DWORD BytesReturned = 0;
	int SessionID = 0;
	int BufSize = (240 * 64 / 8) + 8;
	BYTE* FinalBuffer = (BYTE*)malloc(BufSize);
	int* IntBuf = (INT*)FinalBuffer;
	IntBuf[0] = Session;
	IntBuf[1] = 240 * 64 / 8;
	memcpy(&FinalBuffer[8], Data, 240 * 64 / 8);
	DeviceIoControl(handle, IOCTL_SESSION_SEND_FRAME, FinalBuffer, BufSize, NULL, 0, &BytesReturned, NULL);
	free(FinalBuffer);
}

int main()
{
	HDEVINFO                         hDevInfo;
	SP_DEVICE_INTERFACE_DATA         DevIntfData;
	PSP_DEVICE_INTERFACE_DETAIL_DATA DevIntfDetailData;
	SP_DEVINFO_DATA                  DevData;

	DWORD dwSize, dwType, dwMemberIdx;
	HKEY hKey;
	BYTE lpData[1024];

	hDevInfo = SetupDiGetClassDevs(
		&GUID_DEVINTERFACE_GX3, NULL, 0, DIGCF_DEVICEINTERFACE | DIGCF_PRESENT);

	if (hDevInfo != INVALID_HANDLE_VALUE)
	{
		DevIntfData.cbSize = sizeof(SP_DEVICE_INTERFACE_DATA);
		dwMemberIdx = 0;

		SetupDiEnumDeviceInterfaces(hDevInfo, NULL, &GUID_DEVINTERFACE_GX3,
			dwMemberIdx, &DevIntfData);

		while (GetLastError() != ERROR_NO_MORE_ITEMS)
		{
			DevData.cbSize = sizeof(DevData);

			SetupDiGetDeviceInterfaceDetail(
				hDevInfo, &DevIntfData, NULL, 0, &dwSize, NULL);

			DevIntfDetailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, dwSize);
			DevIntfDetailData->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);

			if (SetupDiGetDeviceInterfaceDetail(hDevInfo, &DevIntfData,
				DevIntfDetailData, dwSize, &dwSize, &DevData))
			{
				wprintf(_T("Found a GX path = '%s'\n"), DevIntfDetailData->DevicePath);
				HANDLE h = CreateFile(DevIntfDetailData->DevicePath, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_DELETE | FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, 0);
				if (h)
				{
					wprintf(_T("Opened GX \n"));
					int Session = CreateSession(h, "Test");
					while (true)
					{
						BYTE Buffer[240 * 64 / 8];
						for (int i = 0; i < 240 * 64 / 8; i++)
							Buffer[i] = rand() & 0xff;
						SendFrame(h, Session, Buffer);
					}

					CloseHandle(h);
				}
			}

			HeapFree(GetProcessHeap(), 0, DevIntfDetailData);
			SetupDiEnumDeviceInterfaces(
				hDevInfo, NULL, &GUID_DEVINTERFACE_GX3, ++dwMemberIdx, &DevIntfData);
		}

		SetupDiDestroyDeviceInfoList(hDevInfo);
	}

	return 0;
}

