﻿using System;
using System.Collections.Generic;
 
using System.Text;
using MatrixOrbital.Tools;
using Microsoft.Win32.SafeHandles;

namespace DriverTest
{
    public class GXDriver
    {
        private const uint IOCTL_INDEX = 0x800;
        private const uint FILE_DEVICE_GX3 = 0x65500;
        private uint IOCTL_GX3_GET_METRICS = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_READ_ACCESS);
        private uint IOCTL_RAW_SET_BACKLIGHT = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX + 1, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_WRITE_ACCESS);
        private uint IOCTL_RAW_SET_CONTRAST = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX + 2, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_WRITE_ACCESS);
        private uint IOCTL_RAW_GET_SPLASH = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX + 4, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_READ_ACCESS);
        private uint IOCTL_RAW_GET_FRAME = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX + 5, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_READ_ACCESS);
        private uint IOCTL_RAW_PUT_FRAME = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX + 6, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_WRITE_ACCESS);
        private uint IOCTL_SET_DEFAULT_BACKLIGHT = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX + 7, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_WRITE_ACCESS);
        private uint IOCTL_SET_DEFAULT_CONTRAST = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX + 8, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_WRITE_ACCESS);
        private uint IOCTL_SESSION_CREATE = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX + 9, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_ANY_ACCESS);
        private uint IOCTL_SESSION_CLOSE = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX + 10, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_ANY_ACCESS);
        private uint IOCTL_SESSION_PUT_FRAME = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX + 11, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_WRITE_ACCESS);
        private uint IOCTL_SESSION_SET_CONTRAST = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX + 12, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_WRITE_ACCESS);
        private uint IOCTL_SESSION_SET_BACKLIGHT = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX + 13, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_WRITE_ACCESS);
        private uint IOCTL_SESSION_LIST_SESSIONS = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX + 14, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_READ_ACCESS);
        private uint IOCTL_GET_FW_REV = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX + 15, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_READ_ACCESS);
        private uint IOCTL_GET_SERIAL = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX + 16, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_READ_ACCESS);
        private uint IOCTL_GET_DEFAULT_BACKLIGHT = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX + 17, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_READ_ACCESS);
        private uint IOCTL_GET_DEFAULT_CONTRAST = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX + 18, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_READ_ACCESS);
        private uint IOCTL_START_FWU = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX + 19, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_ANY_ACCESS);
        private uint IOCTL_GET_IR_STATE = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX + 20, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_READ_ACCESS);
        private uint IOCTL_SET_IR_STATE = MatrixOrbital.Tools.ioctl.CTL_CODE(FILE_DEVICE_GX3, IOCTL_INDEX + 21, ioctl.eMethod.METHOD_BUFFERED, ioctl.eFileAccess.FILE_WRITE_ACCESS);
        
        private SafeFileHandle _Handle;
        private string _DevicePath;
        public GXDriver(string DevicePath)
        {
            _Handle = MatrixOrbital.Tools.Native.CreateFile(DevicePath, Native.EFileAccess.GenericRead | Native.EFileAccess.GenericWrite, Native.EFileShare.All, IntPtr.Zero, System.IO.FileMode.Open, Native.EFileAttributes.None, IntPtr.Zero);
            if (_Handle.IsInvalid)
            {
                throw new ArgumentException("Cannot open device");
            }
        }

        public void Close()
        {
            _Handle.Close();
        }

        private void EnsureOpen()
        {
            if (_Handle.IsInvalid)
            {
                while(_Handle.IsInvalid)
                {
                    _Handle = MatrixOrbital.Tools.Native.CreateFile(_DevicePath, Native.EFileAccess.GenericRead | Native.EFileAccess.GenericWrite, Native.EFileShare.All, IntPtr.Zero, System.IO.FileMode.Open, Native.EFileAttributes.None, IntPtr.Zero);
                    if (!_Handle.IsInvalid)
                        break;
                    System.Threading.Thread.Sleep(100);
                }
            }
        }
        
        public void SwitchToFWU()
        {
            EnsureOpen();
            int br = 0;
            bool success = MatrixOrbital.Tools.Native.DeviceIoControl(_Handle, IOCTL_START_FWU, null, 0, null, 0, ref br, null);
        }

        public int GetIRState()
        {
            EnsureOpen();
            int br = 0;
            byte[] outbuf = new byte[1];
            bool success = MatrixOrbital.Tools.Native.DeviceIoControl(_Handle, IOCTL_GET_IR_STATE, null, 0, outbuf, outbuf.Length, ref br, null);
            return outbuf[0];
        }

        public void SetIRState(int state)
        {
            EnsureOpen();
            int br = 0;
            byte[] inbuf = new byte[] { (byte)state };
            bool success = MatrixOrbital.Tools.Native.DeviceIoControl(_Handle, IOCTL_SET_IR_STATE, inbuf,inbuf.Length, null, 0, ref br, null);
        }
        public void SetBackLight(int r, int g, int b)
        {
            EnsureOpen();
            byte[] inbuf = new byte[3];
            int br = 0;
            inbuf[0] = (byte)r;
            inbuf[1] = (byte)g;
            inbuf[2] = (byte)b;
            bool success = MatrixOrbital.Tools.Native.DeviceIoControl(_Handle, IOCTL_RAW_SET_BACKLIGHT, inbuf, inbuf.Length, null, 0, ref br, null);
        }

        public string GetFWrev()
        {
            EnsureOpen();
            byte[] inbuf = new byte[0];
            byte[] outbuf = new byte[256];
            int br = 0;
            bool success = MatrixOrbital.Tools.Native.DeviceIoControl(_Handle, IOCTL_GET_FW_REV, inbuf, inbuf.Length,outbuf,outbuf.Length, ref br, null);
            return System.Text.Encoding.Unicode.GetString(outbuf, 2, br-2);
        }

        public string GetSerial()
        {
            EnsureOpen();
            byte[] inbuf = new byte[0];
            byte[] outbuf = new byte[256];
            int br = 0;
            bool success = MatrixOrbital.Tools.Native.DeviceIoControl(_Handle, IOCTL_GET_SERIAL, inbuf, inbuf.Length, outbuf, outbuf.Length, ref br, null);
            return System.Text.Encoding.Unicode.GetString(outbuf, 2, br - 2);
        }

        public void SetBackLight(int SessionID,int r, int g, int b)
        {
            EnsureOpen();
            byte[] inbuf = new byte[7];
            int br = 0;
            byte[] sess = BitConverter.GetBytes(SessionID);
            Array.Copy(sess, inbuf, 4);
            inbuf[4] = (byte)r;
            inbuf[5] = (byte)g;
            inbuf[6] = (byte)b;
            bool success = MatrixOrbital.Tools.Native.DeviceIoControl(_Handle, IOCTL_SESSION_SET_BACKLIGHT, inbuf, inbuf.Length, null, 0, ref br, null);
        }

        public void GetDefaultBackLight(out byte r, out byte g, out byte b)
        {
            EnsureOpen();
            byte[] inbuf = new byte[0];
            byte[] outbuf = new byte[3];
            int br = 0;
            bool success = MatrixOrbital.Tools.Native.DeviceIoControl(_Handle, IOCTL_GET_DEFAULT_BACKLIGHT, inbuf, inbuf.Length,outbuf,outbuf.Length, ref br, null);
            r = outbuf[0];
            g = outbuf[1];
            b = outbuf[2];
        }

        public void SetDefaultBackLight(byte r, byte g, byte b)
        {
            EnsureOpen();
            byte[] inbuf = new byte[3];
            byte[] outbuf = new byte[0];
            inbuf[0] = r;
            inbuf[1] = g;
            inbuf[2] = b;
            int br = 0;
            bool success = MatrixOrbital.Tools.Native.DeviceIoControl(_Handle, IOCTL_SET_DEFAULT_BACKLIGHT, inbuf, inbuf.Length, outbuf, outbuf.Length, ref br, null);
        }


        public void SendFrame(int SessionID, byte[] Frame)
        {
            byte[] inbuf = new byte[Frame.Length + 8];
            Array.Copy(BitConverter.GetBytes(SessionID), inbuf, 4);
            Array.Copy(BitConverter.GetBytes(Frame.Length),0, inbuf, 4,4);
            Array.Copy(Frame,0, inbuf, 8, Frame.Length);
            int br = 0;
            bool success = MatrixOrbital.Tools.Native.DeviceIoControl(_Handle, IOCTL_SESSION_PUT_FRAME, inbuf, inbuf.Length, null, 0, ref br, null);
        }

        public bool CloseSession(int SessionID)
        {
            byte[] inbuf = BitConverter.GetBytes(SessionID);
            int br = 0;
            byte[] Outbuf = new byte[4];
            bool success = MatrixOrbital.Tools.Native.DeviceIoControl(_Handle, IOCTL_SESSION_CLOSE, inbuf, inbuf.Length, Outbuf, Outbuf.Length, ref br, null);
            if ((success) && (br == 4))
            {
                int Sess = BitConverter.ToInt32(Outbuf, 0);
                return (Sess != 0);
            }
            return false;
        }
        public byte[] GetSplashScreen()
        {
            EnsureOpen();
            byte[] inbuf = new byte[0];
            byte[] outbuf = new byte[4096];
            int br = 0;
            bool success = MatrixOrbital.Tools.Native.DeviceIoControl(_Handle, IOCTL_RAW_GET_SPLASH, inbuf, inbuf.Length, outbuf, outbuf.Length, ref br, null);
            return outbuf;
        }

        public int CreateSession(string SessionName)
        {
            byte[] inbuf = new byte[System.Text.Encoding.UTF8.GetByteCount(SessionName)+1];
            Array.Copy(System.Text.Encoding.UTF8.GetBytes(SessionName),inbuf,inbuf.Length-1);
            int br = 0;
            byte[] Outbuf = new byte[4];
            bool success = MatrixOrbital.Tools.Native.DeviceIoControl(_Handle, IOCTL_SESSION_CREATE, inbuf, inbuf.Length, Outbuf, Outbuf.Length, ref br, null);
            if ((success) && (br == 4))
            {
                int Sess = BitConverter.ToInt32(Outbuf, 0);
                return Sess;
            }
            return -1;
        }


        internal void SetContrast(int SessionID, byte contrast)
        {
            EnsureOpen();
            byte[] inbuf = new byte[5];
            int br = 0;
            byte[] sess = BitConverter.GetBytes(SessionID);
            Array.Copy(sess, inbuf, 4);
            inbuf[4] = (byte)contrast;
            bool success = MatrixOrbital.Tools.Native.DeviceIoControl(_Handle, IOCTL_SESSION_SET_CONTRAST, inbuf, inbuf.Length, null, 0, ref br, null);
        }

        internal void SetDefaultContrast(byte contrast)
        {
            EnsureOpen();
            byte[] inbuf = new byte[1];
            int br = 0;
            inbuf[0] = (byte)contrast;
            bool success = MatrixOrbital.Tools.Native.DeviceIoControl(_Handle, IOCTL_SET_DEFAULT_CONTRAST, inbuf, inbuf.Length, null, 0, ref br, null);
        }
    }
}
